/* Copyright (c) 2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.main;

import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ComponentEvent;
import java.awt.event.ComponentListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

@SuppressWarnings("serial")
public class Chunk3DView extends JDialog {
    
    private Canvas3D canvas;
    private JSlider yawSlider;
    private JSlider zoomSlider;
    
    int x0, y0;
    
    public Chunk3DView(Chunky chunky, List<byte[]> blockData) {
        super(chunky.frame);
        
        JPanel panel = new JPanel(new BorderLayout());
        getContentPane().add(panel);
        
        canvas = new Canvas3D();
        panel.add(canvas, BorderLayout.CENTER);

        JPanel controls = new JPanel(new BorderLayout());
        
        JPanel sliders = new JPanel(new BorderLayout());
        yawSlider = new JSlider(0, 100);
        sliders.add(yawSlider, BorderLayout.NORTH);
        
        JSlider pitchSlider = new JSlider(0, 100);
        pitchSlider.setValue(10);
        sliders.add(pitchSlider, BorderLayout.CENTER);
        
        zoomSlider = new JSlider(100, 300);
        zoomSlider.setValue(300);
        sliders.add(zoomSlider, BorderLayout.SOUTH);
        
        JButton goBtn = new JButton("Go");
        
        controls.add(sliders, BorderLayout.CENTER);
        controls.add(goBtn, BorderLayout.EAST);
        panel.add(controls, BorderLayout.SOUTH);
        
        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setModalityType(Dialog.ModalityType.MODELESS);
        
        setTitle("3D Chunk View");
        pack();
        setLocationRelativeTo(chunky);
        setVisible(true);
        requestFocus();
        
        canvas.init();
        
        final RenderController renderController = new RenderController(canvas, blockData);
        renderController.start();
        
        yawSlider.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				JSlider source = (JSlider) e.getSource();
				renderController.setYaw(source.getValue());
			}
		});
        
        pitchSlider.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                JSlider source = (JSlider) e.getSource();
                renderController.setPitch(source.getValue());
            }
        });
        
        zoomSlider.addChangeListener(new ChangeListener() {
            @Override
            public void stateChanged(ChangeEvent e) {
                JSlider source = (JSlider) e.getSource();
                renderController.setZoom(source.getValue());
            }
        });
        
        goBtn.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent arg0) {
				renderController.renderSlow();
			}
		});
        
        addWindowListener(new WindowListener() {
            @Override
            public void windowOpened(WindowEvent e) {
            }
            @Override
            public void windowIconified(WindowEvent e) {
            }
            @Override
            public void windowDeiconified(WindowEvent e) {
            }
            @Override
            public void windowDeactivated(WindowEvent e) {
            }
            @Override
            public void windowClosing(WindowEvent e) {
                renderController.interrupt();
            }
            @Override
            public void windowClosed(WindowEvent e) {
            }
            @Override
            public void windowActivated(WindowEvent e) {
            }
        });
        
        addComponentListener(new ComponentListener() {
            @Override
            public void componentShown(ComponentEvent e) {
            }
            @Override
            public void componentResized(ComponentEvent e) {
                renderController.refresh();
            }
            @Override
            public void componentMoved(ComponentEvent e) {
            }
            @Override
            public void componentHidden(ComponentEvent e) {
            }
        });
        
        canvas.addKeyListener(new KeyListener() {
            @Override
            public void keyTyped(KeyEvent e) {
                onKeyDown(e);
            }
            @Override
            public void keyReleased(KeyEvent e) {
            }
            @Override
            public void keyPressed(KeyEvent e) {
                onKeyDown(e);
            }
            private void onKeyDown(KeyEvent e) {
                switch (e.getKeyCode()) {
                case KeyEvent.VK_A:
                case KeyEvent.VK_LEFT:
                    renderController.strafeLeft(10);
                    break;
                case KeyEvent.VK_D:
                case KeyEvent.VK_RIGHT:
                    renderController.strafeRight(10);
                    break;
                case KeyEvent.VK_W:
                case KeyEvent.VK_UP:
                    renderController.moveForward(10);
                    break;
                case KeyEvent.VK_S:
                case KeyEvent.VK_DOWN:
                    renderController.moveBackward(10);
                    break;
                }                
            }
        });
        
        canvas.addMouseMotionListener(new MouseMotionListener() {
            @Override
            public void mouseMoved(MouseEvent e) {
                // TODO Auto-generated method stub
                
            }
            @Override
            public void mouseDragged(MouseEvent e) {
                int dx = e.getX() - x0;
                int dy = e.getY() - y0;
                renderController.rotateView(
                        (float) (Math.PI / 150) * dx,
                        - (float) (Math.PI / 150) * dy);
                x0 = e.getX();
                y0 = e.getY();
            }
        });
        canvas.addMouseListener(new MouseListener() {
            @Override
            public void mouseReleased(MouseEvent e) {
            }
            @Override
            public void mousePressed(MouseEvent e) {
                x0 = e.getX();
                y0 = e.getY();
            }
            @Override
            public void mouseExited(MouseEvent e) {
            }
            @Override
            public void mouseEntered(MouseEvent e) {
            }
            @Override
            public void mouseClicked(MouseEvent e) {
            }
        });
        
        canvas.requestFocusInWindow();
    }
}
