/* Copyright (c) 2010-2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.main;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.image.BufferStrategy;

import se.llbit.chunky.world.ChunkView;
import se.llbit.chunky.world.World;
import se.llbit.chunky.world.WorldRenderer;

/**
 * @author Jesper Öqvist (jesper@llbit.se)
 */
@SuppressWarnings("serial")
public class Minimap extends Canvas {

	private RenderBuffer renderBuffer;
	private BufferStrategy strategy;
	private Font dirFont = new Font("Sans serif", Font.BOLD, 11);
	private Chunky chunky;

	public Minimap(Chunky parent) {
		this.chunky = parent;
		setBackground(Color.white);
		setPreferredSize(new Dimension(150, 150));
		setMinimumSize(new Dimension(100, 100));
		setIgnoreRepaint(true);
		
		renderBuffer = new RenderBuffer(getView(0, 0));
		
		addMouseListener(new MouseListener() {
			@Override
			public void mouseReleased(MouseEvent e) {
			}
			@Override
			public void mousePressed(MouseEvent e) {
				chunky.moveView(e.getY()-getHeight()/2, -e.getX()+getWidth()/2);
			}
			@Override
			public void mouseExited(MouseEvent e) {
			}
			@Override
			public void mouseEntered(MouseEvent e) {
			}
			@Override
			public void mouseClicked(MouseEvent e) {
			}
		});
	}
	
	private ChunkView getView(double x0, double z0) {
		int width = getWidth();
		int height = getHeight();
		double mw = width / 2.0;
		double mh = height/ 2.0;
		return new ChunkView(x0 - mh, x0 + mh, z0 - mw, z0 + mw, width, height, 1);
	}

	public void init() {
		createBufferStrategy(2);
		strategy = getBufferStrategy();
	}
	
	public void refresh(WorldRenderer renderer, World world, double x0, double z0, ChunkView mainView) {
		
		ChunkView mapView = getView(x0, z0);
		
		Graphics g = strategy.getDrawGraphics();
		g.clearRect(0, 0, mapView.width, mapView.height);
		
		renderBuffer.setView(mapView, null, 0);
		renderer.renderMinimap(world, renderBuffer);
		renderBuffer.renderBuffered(g);
		
		g.setColor(Color.orange);
		g.drawRect(
				(int) Math.round(mainView.z0 - mapView.iz0)-5,// TODO these annoying adjustments should not be needed
				(int) Math.round(mainView.x0 - mapView.ix0)-3,// TODO these annoying adjustments should not be needed
				(int) (mainView.width / (float) mainView.scale),
				(int) (mainView.height / (float) mainView.scale));
		g.setFont(dirFont);
		g.setColor(Color.red);
		g.drawString("N", mapView.width / 2, 10);
		
		g.setColor(Color.black);
		g.drawString(world.levelName(), 10, mapView.height-10);
		
		g.dispose();
		strategy.show();
	}

	public synchronized void flushCache() {
		renderBuffer.flushCache();
	}

}
