/* Copyright (c) 2010-2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.main;

import java.awt.Dialog;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileFilter;

import javax.swing.DefaultListModel;
import javax.swing.JDialog;
import javax.swing.JFileChooser;
import javax.swing.JList;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;

import se.llbit.chunky.world.World;

/**
 * A world selection dialog box. It displays six buttons: one for loading each of the
 * five standard worlds in the Minecraft save directory and a sixth for loading a
 * world outside the standard save directory. 
 * 
 * @author Jesper Öqvist (jesper@llbit.se)
 */
@SuppressWarnings("serial")
public class WorldSelector extends JDialog implements ListSelectionListener {

	private Chunky chunky;
	private DefaultListModel listModel;
	private JList worldList;

	/**
	 * Creates new form WorldSelector 
	 * @param chunky
	 */
	public WorldSelector(Chunky chunky) {
		super(chunky.frame);
		this.chunky = chunky;
		
		initComponents();

		setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setModalityType(Dialog.ModalityType.APPLICATION_MODAL);
		
		setTitle(Messages.getString("WorldSelector.0")); //$NON-NLS-1$
		pack();
		setLocationRelativeTo(chunky.frame);
		setVisible(true);
		requestFocus();
	}

	private void initComponents() {

		File saveDir = Chunky.getSavesDirectory();
		
		jLabel1 = new javax.swing.JLabel();
		browseBtn = new javax.swing.JButton();
		jSeparator1 = new javax.swing.JSeparator();

		listModel = new DefaultListModel();
		worldList = new JList(listModel);
		worldList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);
		worldList.addListSelectionListener(this);
		File[] worldDirs = saveDir.listFiles(new FileFilter() {
			@Override
			public boolean accept(File file) {
				return file.isDirectory();
			}
		});
		for (File worldDir : worldDirs) {
			listModel.addElement(new World(worldDir));
		}
		JScrollPane scrollPane = new JScrollPane(worldList);
		
		jLabel1.setText(Messages.getString("WorldSelector.1")); //$NON-NLS-1$

		browseBtn.setText(Messages.getString("WorldSelector.2")); //$NON-NLS-1$
		browseBtn.addActionListener(new ActionListener() {
			
			@Override
			public void actionPerformed(ActionEvent arg0) {
				JFileChooser fileChooser = new JFileChooser(Chunky.getSavesDirectory());
				fileChooser.setDialogTitle(Messages.getString("WorldSelector.0")); //$NON-NLS-1$
				fileChooser.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);
				int result = fileChooser.showOpenDialog(null);
				if (result == JFileChooser.APPROVE_OPTION) {
					chunky.loadWorld(new World(fileChooser.getSelectedFile()));
					WorldSelector.this.dispose();
				}
			}
		});

		javax.swing.GroupLayout layout = new javax.swing.GroupLayout(getContentPane());
		getContentPane().setLayout(layout);
		layout.setHorizontalGroup(
			layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
			.addGroup(layout.createSequentialGroup()
				.addContainerGap()
				.addGroup(layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
					.addComponent(jLabel1, javax.swing.GroupLayout.DEFAULT_SIZE, 231, Short.MAX_VALUE)
					.addComponent(scrollPane, javax.swing.GroupLayout.DEFAULT_SIZE, 231, Short.MAX_VALUE)
					.addComponent(browseBtn, javax.swing.GroupLayout.DEFAULT_SIZE, 231, Short.MAX_VALUE)
					.addComponent(jSeparator1, javax.swing.GroupLayout.Alignment.TRAILING, javax.swing.GroupLayout.DEFAULT_SIZE, 231, Short.MAX_VALUE))
				.addContainerGap())
		);
		layout.setVerticalGroup(
			layout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
			.addGroup(layout.createSequentialGroup()
				.addContainerGap()
				.addComponent(jLabel1)
				.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
				.addComponent(scrollPane)
				.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.UNRELATED)
				.addComponent(jSeparator1, javax.swing.GroupLayout.PREFERRED_SIZE, 10, javax.swing.GroupLayout.PREFERRED_SIZE)
				.addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
				.addComponent(browseBtn)
				.addContainerGap(javax.swing.GroupLayout.DEFAULT_SIZE, Short.MAX_VALUE))
		);

		pack();
	}

	private javax.swing.JButton browseBtn;
	private javax.swing.JLabel jLabel1;
	private javax.swing.JSeparator jSeparator1;
	
	@Override
	public void valueChanged(ListSelectionEvent event) {
		int selected = event.getFirstIndex();
		World world = (World) listModel.get(selected);
		chunky.loadWorld(world);
		WorldSelector.this.dispose();
	}
}
