package se.llbit.chunky.meta;

import javax.vecmath.Vector3f;

public class Ray {
	
	public Vector3f d = new Vector3f();
	public Vector3f o = new Vector3f();
	
	public Vector3f intersection = new Vector3f();
	public Vector3f intersectionNormal = new Vector3f();
	
	public float[] color = new float[3];
	
	public int currentMaterial;
	
	public int rayDepth;
	
	public float attenuation;
	
	private Ray() {
	}
	
	/**
	 * set default values
	 */
	public void setDefault() {
        currentMaterial = -1;
        rayDepth = 4;
        attenuation = 1;
        color[0] = 0;
        color[1] = 0;
        color[2] = 0;
    }
	
	public void set(Ray other) {
	    currentMaterial = other.currentMaterial;
        rayDepth = other.rayDepth-1;
        attenuation = other.attenuation;
        color[0] = other.color[0];
        color[1] = other.color[1];
        color[2] = other.color[2];
	}
	
	public static class RayPool {
	    private int limit = 10;
	    private int size = 0;
    	private Ray[] pool = new Ray[limit];
    
    	public final Ray get(Ray other) {
    	    Ray ray;
    	    if (size == 0) {
    	        ray = new Ray();
    	    } else {
    	        ray = pool[--size];
    	    }
    	    ray.set(other);
            return ray;
        }
    	
        public final Ray get() {
            Ray ray;
            if (size == 0) {
                ray = new Ray();
            } else {
                ray = pool[--size];
            }
            ray.setDefault();
            return ray;
        }
        
        public final void dispose(Ray ray) {
            if (size == limit) {
                int newLimit = limit * 2;
                Ray[] newPool = new Ray[newLimit];
                System.arraycopy(pool, 0, newPool, 0, limit);
                limit = newLimit;
                pool = newPool;
            }
            pool[size++] = ray;
        }
	}

}
