/* Copyright (c) 2010-2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.world;

import java.awt.image.BufferedImage;

import se.llbit.resources.ImageLoader;
import se.llbit.util.ImageTools;

/**
 * @author Jesper Öqvist <jesper.oqvist@cs.lth.se>
 */
@SuppressWarnings("javadoc")
public enum Block {
	AIR				(0x00, "Air", "air", 1.000293f),
	STONE			(0x01, "Stone", "stone", true),
	GRASS			(0x02, "Grass", "grass", true),
	DIRT			(0x03, "Dirt", "dirt", true),
	COBBLESTONE		(0x04, "Cobblestone", "cobblestone", true),
	WOODENPLANKS	(0x05, "Wooden Planks", "wooden-planks", true),
	SAPLING			(0x06, "Sapling", "sapling", false),
	BEDROCK			(0x07, "Bedrock", "bedrock", true),
	WATER			(0x08, "Water", "water", false),
	STATIONARYWATER	(0x09, "Stationary Water", "water", false),
	LAVA			(0x0A, "Lava", "lava", true),
	STATIONARYLAVA	(0x0B, "Stationary Lava", "lava", true),
	SAND			(0x0C, "Sand", "sand", true),
	GRAVEL			(0x0D, "Gravel", "gravel", true),
	GOLDORE			(0x0E, "Gold Ore", "gold-ore", true),
	IRONORE			(0x0F, "Iron Ore", "iron-ore", true),
	COALORE			(0x10, "Coal Ore", "coal-ore", true),
	WOOD			(0x11, "Wood Log", "wood", true),
	LEAVES			(0x12, "Leaves", "leaves", false),
	SPONGE			(0x13, "Sponge", "sponge", true),
	GLASS			(0x14, "Glass", "glass", false),
	LAPISLAZULIORE	(0x15, "Lapis Lazuli Ore", "lapis-lazuli-ore", true),
	LAPISLAZULIBLOCK(0x16, "Lapis Lazuli Block", "lapis-lazuli-block", true),
	DISPENSER		(0x17, "Dispenser", "dispenser", true),
	SANDSTONE		(0x18, "Sandstone", "sandstone", true),
	NOTEBLOCK		(0x19, "Note Block", "note-block", true),
	BED				(0x1A, "Bed", "bed", false),
	POWEREDRAIL		(0x1B, "Powered Rail", "powered-rail", false),
	DETECTORRAIL	(0x1C, "Detector Rail", "detector-rail", false),
	STICKYPISTON	(0x1D, "Sticky Piston", "piston", true),
	COBWEB			(0x1E, "Cobweb", "cobweb", false),
	TALLGRASS		(0x1F, "Tall Grass", "tall-grass", false),
	DEADBUSH		(0x20, "Dead Bush", "dead-bush", false),
	PISTON			(0x21, "Piston", "piston", true),
	PISTONEXTENSION	(0x22, "Piston Extension", "piston-extension", false),
	WOOL			(0x23, "Wool", "wool", true),
	MOVEDBYPISTON	(0x24, "Block moved by Piston", "unknown", false),
	YELLOWFLOWER	(0x25, "Yellow Flower", "yellow-flower", false),
	REDROSE			(0x26, "Red Rose", "rose", false),
	BROWNMUSHROOM	(0x27, "Brown Mushroom", "brown-mushroom", false),
	REDMUSHROOM		(0x28, "Red Mushroom", "red-mushroom", false),
	GOLDBLOCK		(0x29, "Gold Block", "gold-block", true),
	IRONBLOCK		(0x2A, "Iron Block", "iron-block", true),
	DOUBLESTONESLAB	(0x2B, "Double Stone Slab", "double-stone-slab", true),
	STONESLAB		(0x2C, "Stone Slab", "stone-slab", false),
	BRICKS			(0x2D, "Bricks", "bricks", true),
	TNT				(0x2E, "TNT", "tnt", true),
	BOOKSHELF		(0x2F, "Bookshelf", "bookshelf", true),
	MOSSSTONE		(0x30, "Moss Stone", "moss-stone", true),
	OBSIDIAN		(0x31, "Obsidian", "obsidian", true),
	TORCH			(0x32, "Torch", "torch", false),
	FIRE			(0x33, "Fire", "fire", false),
	MONSTERSPAWNER	(0x34, "Monster Spawner", "spawner", false),
	WOODENSTAIRS	(0x35, "Wooden Stairs", "wooden-stairs", "wooden-planks", false),
	CHEST			(0x36, "Chest", "chest", true),
	REDSTONEWIRE	(0x37, "Redstone Wire", "redstone-wire-off-intersect", false),
	DIAMONDORE		(0x38, "Diamond Ore", "diamond-ore", true),
	DIAMONDBLOCK	(0x39, "Diamond Block", "diamond-block", true),
	WORKBENCH		(0x3A, "Workbench", "workbench", true),
	CROPS			(0x3B, "Crops", "wheat", false),
	SOIL			(0x3C, "Soil", "soil", true),
	FURNACE			(0x3D, "Furnace", "furnace", true),
	BURNINGFURNACE	(0x3E, "Burning Furnace", "furnace-lit", true),
	SIGNPOST		(0x3F, "Sign Post", "sign-post", false),
	WOODENDOOR		(0x40, "Wooden Door", "wooden-door", false),
	LADDER			(0x41, "Ladder", "ladder", false),
	MINECARTTRACKS	(0x42, "Minecart Tracks", "minecart-track", false),
	STONESTAIRS		(0x43, "Stone Stairs", "stone-stairs", "stone", false),
	WALLSIGN		(0x44, "Wall Sign", "wall-sign", false),
	LEVER			(0x45, "Lever", "lever", false),
	STONEPRESSUREPLATE(0x46, "Stone Pressure Plate", "stone-pressure-plate", false),
	IRONDOOR		(0x47, "Iron Door", "iron-door", false),
	WOODENPRESSUREPLATE(0x48, "Wooden Pressure Plate", "wooden-pressure-plate", false),
	REDSTONEORE		(0x49, "Redstone Ore", "redstone-ore", true),
	GLOWINGREDSTONEORE(0x4A, "Glowing Redstone Ore", "redstone-ore", true),
	REDSTONETORCHOFF(0x4B, "Redstone Torch (off)", "redstone-torch-off", false),
	REDSTONETORCHON	(0x4C, "Redstone Torch (on)", "redstone-torch-on", false),
	STONEBUTTON		(0x4D, "Stone Button", "stone-button", false),
	SNOW			(0x4E, "Snow", "snow", false),
	ICE				(0x4F, "Ice", "ice", false),
	SNOWBLOCK		(0x50, "Snow Block", "snow", true),
	CACTUS			(0x51, "Cactus", "cactus", false),
	CLAY			(0x52, "Clay", "clay", true),
	SUGARCANE		(0x53, "Sugar Cane", "sugar-canes", false),
	JUKEBOX			(0x54, "Jukebox", "jukebox", true),
	FENCE			(0x55, "Fence", "fence", false),
	PUMPKIN			(0x56, "Pumpkin", "pumpkin", true),
	NETHERRACK		(0x57, "Netherrack", "netherrack", true),
	SOULSAND		(0x58, "Soul Sand", "soul-sand", true),
	GLOWSTONE		(0x59, "Glowstone", "glowstone", true),
	PORTAL			(0x5A, "Portal", "nether-portal", true),
	JACKOLANTERN	(0x5B, "Jack-O-Lantern", "jack-o-lantern", true),
	CAKE			(0x5C, "Cake Block", "cake", false),
	REDSTONEREPEATEROFF(0x5D, "Redstone Repeater (off)", "redstone-repeater-off", false),
	REDSTONEREPEATERON(0x5E, "Redstone Repeater (on)", "redstone-repeater-on", false),
	LOCKEDCHEST		(0x5F, "Locked Chest", "locked-chest", true),
	TRAPDOOR		(0x60, "Trapdoor", "trapdoor", false),
	HIDDENSILVERFISH(0x61, "Hidden Silverfish", "stone", true),
	STONEBRICKS		(0x62, "Stone Bricks", "stone-bricks", true),
	HUGEBROWNMUSHROOM(0x63, "Huge Brown Mushroom", "huge-brown-mushroom", true),
	HUGEREDMUSHROOM	(0x64, "Huge Red Mushroom", "huge-red-mushroom", true),
	IRONBARS		(0x65, "Iron Bars", "iron-bars", false),
	GLASSPANE		(0x66, "Glass Pane", "glass", false),
	MELON			(0x67, "Melon", "melon", true),
	PUMPKINSTEM		(0x68, "Pumpkin Stem", "pumpkin-stem", false),
	MELONSTEM		(0x69, "Melon Stem", "melon-stem", false),
	VINES			(0x6A, "Vines", "vines", false),
	FENCEGATE		(0x6B, "Fence Gate", "fence-gate", false),
	BRICKSTAIRS		(0x6C, "Brick Stairs", "brick-stairs", false),
	STONEBRICKSTAIRS(0x6D, "Stone Brick Stairs", "stone-brick-stairs", false),
	MYCELIUM		(0x6E, "Mycelium", "mycelium", true),
	LILYPAD			(0x6F, "Lily Pad", "lily-pad", false),
	NETHERBRICK		(0x70, "Nether Brick", "nether-brick", true),
	NETHERBRICKFENCE(0x71, "Nether Brick Fence", "nether-brick-fence", false),
	NETHERBRICKSTAIRS(0x72, "Nether Brick Stairs", "nether-brick-stairs", false),
	NETHERWART		(0x73, "Nether Wart", "nether-wart", false),
	ENCHNATMENTTABLE(0x74, "Enchantment Table", "enchantment-table", false),
	BREWINGSTAND	(0x75, "Brewing Stand", "brewing-stand", false),
	CAULDRON		(0x76, "Cauldron", "cauldron", false),
	ENDPORTAL		(0x77, "End Portal", "end-portal", false),
	ENDPORTALFRAME	(0x78, "End Portal Frame", "end-portal-frame", true),
	ENDSTONE		(0x79, "End Stone", "end-stone", true),
	DRAGONEGG		(0x7A, "Dragon Egg", "dragon-egg", false),
	REDSTONELAMPOFF	(0x7B, "Redstone Lamp (off)", "redstone-lamp-off", true),
	REDSTONELAMPON	(0x7C, "Redstone Lamp (on)", "redstone-lamp-on", true),
	UNKNOWN0x7D		(0x7D, "Unknown Block 0x7D", "unknown", false),
	UNKNOWN0x7E		(0x7E, "Unknown Block 0x7E", "unknown", false),
	UNKNOWN0x7F		(0x7F, "Unknown Block 0x7F", "unknown", false),
	UNKNOWN0x80		(0x80, "Unknown Block 0x80", "unknown", false),
	UNKNOWN0x81		(0x81, "Unknown Block 0x81", "unknown", false),
	UNKNOWN0x82		(0x82, "Unknown Block 0x82", "unknown", false),
	UNKNOWN0x83		(0x83, "Unknown Block 0x83", "unknown", false),
	UNKNOWN0x84		(0x84, "Unknown Block 0x84", "unknown", false),
	UNKNOWN0x85		(0x85, "Unknown Block 0x85", "unknown", false),
	UNKNOWN0x86		(0x86, "Unknown Block 0x86", "unknown", false),
	UNKNOWN0x87		(0x87, "Unknown Block 0x87", "unknown", false),
	UNKNOWN0x88		(0x88, "Unknown Block 0x88", "unknown", false),
	UNKNOWN0x89		(0x89, "Unknown Block 0x89", "unknown", false),
	UNKNOWN0x8A		(0x8A, "Unknown Block 0x8A", "unknown", false),
	UNKNOWN0x8B		(0x8B, "Unknown Block 0x8B", "unknown", false),
	UNKNOWN0x8C		(0x8C, "Unknown Block 0x8C", "unknown", false),
	UNKNOWN0x8D		(0x8D, "Unknown Block 0x8D", "unknown", false),
	UNKNOWN0x8E		(0x8E, "Unknown Block 0x8E", "unknown", false),
	UNKNOWN0x8F		(0x8F, "Unknown Block 0x8F", "unknown", false),
	UNKNOWN0x90		(0x90, "Unknown Block 0x90", "unknown", false),
	UNKNOWN0x91		(0x91, "Unknown Block 0x91", "unknown", false),
	UNKNOWN0x92		(0x92, "Unknown Block 0x92", "unknown", false),
	UNKNOWN0x93		(0x93, "Unknown Block 0x93", "unknown", false),
	UNKNOWN0x94		(0x94, "Unknown Block 0x94", "unknown", false),
	UNKNOWN0x95		(0x95, "Unknown Block 0x95", "unknown", false),
	UNKNOWN0x96		(0x96, "Unknown Block 0x96", "unknown", false),
	UNKNOWN0x97		(0x97, "Unknown Block 0x97", "unknown", false),
	UNKNOWN0x98		(0x98, "Unknown Block 0x98", "unknown", false),
	UNKNOWN0x99		(0x99, "Unknown Block 0x99", "unknown", false),
	UNKNOWN0x9A		(0x9A, "Unknown Block 0x9A", "unknown", false),
	UNKNOWN0x9B		(0x9B, "Unknown Block 0x9B", "unknown", false),
	UNKNOWN0x9C		(0x9C, "Unknown Block 0x9C", "unknown", false),
	UNKNOWN0x9D		(0x9D, "Unknown Block 0x9D", "unknown", false),
	UNKNOWN0x9E		(0x9E, "Unknown Block 0x9E", "unknown", false),
	UNKNOWN0x9F		(0x9F, "Unknown Block 0x9F", "unknown", false),
	UNKNOWN0xA0		(0xA0, "Unknown Block 0xA0", "unknown", false),
	UNKNOWN0xA1		(0xA1, "Unknown Block 0xA1", "unknown", false),
	UNKNOWN0xA2		(0xA2, "Unknown Block 0xA2", "unknown", false),
	UNKNOWN0xA3		(0xA3, "Unknown Block 0xA3", "unknown", false),
	UNKNOWN0xA4		(0xA4, "Unknown Block 0xA4", "unknown", false),
	UNKNOWN0xA5		(0xA5, "Unknown Block 0xA5", "unknown", false),
	UNKNOWN0xA6		(0xA6, "Unknown Block 0xA6", "unknown", false),
	UNKNOWN0xA7		(0xA7, "Unknown Block 0xA7", "unknown", false),
	UNKNOWN0xA8		(0xA8, "Unknown Block 0xA8", "unknown", false),
	UNKNOWN0xA9		(0xA9, "Unknown Block 0xA9", "unknown", false),
	UNKNOWN0xAA		(0xAA, "Unknown Block 0xAA", "unknown", false),
	UNKNOWN0xAB		(0xAB, "Unknown Block 0xAB", "unknown", false),
	UNKNOWN0xAC		(0xAC, "Unknown Block 0xAC", "unknown", false),
	UNKNOWN0xAD		(0xAD, "Unknown Block 0xAD", "unknown", false),
	UNKNOWN0xAE		(0xAE, "Unknown Block 0xAE", "unknown", false),
	UNKNOWN0xAF		(0xAF, "Unknown Block 0xAF", "unknown", false),
	UNKNOWN0xB0		(0xB0, "Unknown Block 0xB0", "unknown", false),
	UNKNOWN0xB1		(0xB1, "Unknown Block 0xB1", "unknown", false),
	UNKNOWN0xB2		(0xB2, "Unknown Block 0xB2", "unknown", false),
	UNKNOWN0xB3		(0xB3, "Unknown Block 0xB3", "unknown", false),
	UNKNOWN0xB4		(0xB4, "Unknown Block 0xB4", "unknown", false),
	UNKNOWN0xB5		(0xB5, "Unknown Block 0xB5", "unknown", false),
	UNKNOWN0xB6		(0xB6, "Unknown Block 0xB6", "unknown", false),
	UNKNOWN0xB7		(0xB7, "Unknown Block 0xB7", "unknown", false),
	UNKNOWN0xB8		(0xB8, "Unknown Block 0xB8", "unknown", false),
	UNKNOWN0xB9		(0xB9, "Unknown Block 0xB9", "unknown", false),
	UNKNOWN0xBA		(0xBA, "Unknown Block 0xBA", "unknown", false),
	UNKNOWN0xBB		(0xBB, "Unknown Block 0xBB", "unknown", false),
	UNKNOWN0xBC		(0xBC, "Unknown Block 0xBC", "unknown", false),
	UNKNOWN0xBD		(0xBD, "Unknown Block 0xBD", "unknown", false),
	UNKNOWN0xBE		(0xBE, "Unknown Block 0xBE", "unknown", false),
	UNKNOWN0xBF		(0xBF, "Unknown Block 0xBF", "unknown", false),
	UNKNOWN0xC0		(0xC0, "Unknown Block 0xC0", "unknown", false),
	UNKNOWN0xC1		(0xC1, "Unknown Block 0xC1", "unknown", false),
	UNKNOWN0xC2		(0xC2, "Unknown Block 0xC2", "unknown", false),
	UNKNOWN0xC3		(0xC3, "Unknown Block 0xC3", "unknown", false),
	UNKNOWN0xC4		(0xC4, "Unknown Block 0xC4", "unknown", false),
	UNKNOWN0xC5		(0xC5, "Unknown Block 0xC5", "unknown", false),
	UNKNOWN0xC6		(0xC6, "Unknown Block 0xC6", "unknown", false),
	UNKNOWN0xC7		(0xC7, "Unknown Block 0xC7", "unknown", false),
	UNKNOWN0xC8		(0xC8, "Unknown Block 0xC8", "unknown", false),
	UNKNOWN0xC9		(0xC9, "Unknown Block 0xC9", "unknown", false),
	UNKNOWN0xCA		(0xCA, "Unknown Block 0xCA", "unknown", false),
	UNKNOWN0xCB		(0xCB, "Unknown Block 0xCB", "unknown", false),
	UNKNOWN0xCC		(0xCC, "Unknown Block 0xCC", "unknown", false),
	UNKNOWN0xCD		(0xCD, "Unknown Block 0xCD", "unknown", false),
	UNKNOWN0xCE		(0xCE, "Unknown Block 0xCE", "unknown", false),
	UNKNOWN0xCF		(0xCF, "Unknown Block 0xCF", "unknown", false),
	UNKNOWN0xD0		(0xD0, "Unknown Block 0xD0", "unknown", false),
	UNKNOWN0xD1		(0xD1, "Unknown Block 0xD1", "unknown", false),
	UNKNOWN0xD2		(0xD2, "Unknown Block 0xD2", "unknown", false),
	UNKNOWN0xD3		(0xD3, "Unknown Block 0xD3", "unknown", false),
	UNKNOWN0xD4		(0xD4, "Unknown Block 0xD4", "unknown", false),
	UNKNOWN0xD5		(0xD5, "Unknown Block 0xD5", "unknown", false),
	UNKNOWN0xD6		(0xD6, "Unknown Block 0xD6", "unknown", false),
	UNKNOWN0xD7		(0xD7, "Unknown Block 0xD7", "unknown", false),
	UNKNOWN0xD8		(0xD8, "Unknown Block 0xD8", "unknown", false),
	UNKNOWN0xD9		(0xD9, "Unknown Block 0xD9", "unknown", false),
	UNKNOWN0xDA		(0xDA, "Unknown Block 0xDA", "unknown", false),
	UNKNOWN0xDB		(0xDB, "Unknown Block 0xDB", "unknown", false),
	UNKNOWN0xDC		(0xDC, "Unknown Block 0xDC", "unknown", false),
	UNKNOWN0xDD		(0xDD, "Unknown Block 0xDD", "unknown", false),
	UNKNOWN0xDE		(0xDE, "Unknown Block 0xDE", "unknown", false),
	UNKNOWN0xDF		(0xDF, "Unknown Block 0xDF", "unknown", false),
	UNKNOWN0xE0		(0xE0, "Unknown Block 0xE0", "unknown", false),
	UNKNOWN0xE1		(0xE1, "Unknown Block 0xE1", "unknown", false),
	UNKNOWN0xE2		(0xE2, "Unknown Block 0xE2", "unknown", false),
	UNKNOWN0xE3		(0xE3, "Unknown Block 0xE3", "unknown", false),
	UNKNOWN0xE4		(0xE4, "Unknown Block 0xE4", "unknown", false),
	UNKNOWN0xE5		(0xE5, "Unknown Block 0xE5", "unknown", false),
	UNKNOWN0xE6		(0xE6, "Unknown Block 0xE6", "unknown", false),
	UNKNOWN0xE7		(0xE7, "Unknown Block 0xE7", "unknown", false),
	UNKNOWN0xE8		(0xE8, "Unknown Block 0xE8", "unknown", false),
	UNKNOWN0xE9		(0xE9, "Unknown Block 0xE9", "unknown", false),
	UNKNOWN0xEA		(0xEA, "Unknown Block 0xEA", "unknown", false),
	UNKNOWN0xEB		(0xEB, "Unknown Block 0xEB", "unknown", false),
	UNKNOWN0xEC		(0xEC, "Unknown Block 0xEC", "unknown", false),
	UNKNOWN0xED		(0xED, "Unknown Block 0xED", "unknown", false),
	UNKNOWN0xEE		(0xEE, "Unknown Block 0xEE", "unknown", false),
	UNKNOWN0xEF		(0xEF, "Unknown Block 0xEF", "unknown", false),
	UNKNOWN0xF0		(0xF0, "Unknown Block 0xF0", "unknown", false),
	UNKNOWN0xF1		(0xF1, "Unknown Block 0xF1", "unknown", false),
	UNKNOWN0xF2		(0xF2, "Unknown Block 0xF2", "unknown", false),
	UNKNOWN0xF3		(0xF3, "Unknown Block 0xF3", "unknown", false),
	UNKNOWN0xF4		(0xF4, "Unknown Block 0xF4", "unknown", false),
	UNKNOWN0xF5		(0xF5, "Unknown Block 0xF5", "unknown", false),
	UNKNOWN0xF6		(0xF6, "Unknown Block 0xF6", "unknown", false),
	UNKNOWN0xF7		(0xF7, "Unknown Block 0xF7", "unknown", false),
	UNKNOWN0xF8		(0xF8, "Unknown Block 0xF8", "unknown", false),
	UNKNOWN0xF9		(0xF9, "Unknown Block 0xF9", "unknown", false),
	UNKNOWN0xFA		(0xFA, "Unknown Block 0xFA", "unknown", false),
	UNKNOWN0xFB		(0xFB, "Unknown Block 0xFB", "unknown", false),
	UNKNOWN0xFC		(0xFC, "Unknown Block 0xFC", "unknown", false),
	UNKNOWN0xFD		(0xFD, "Unknown Block 0xFD", "unknown", false),
	UNKNOWN0xFE		(0xFE, "Unknown Block 0xFE", "unknown", false),
	UNKNOWN0xFF		(0xFF, "Unknown Block 0xFF", "unknown", false);

	public final int id;
	public final float refractionIndex;
	private final String name;
	private final boolean solid;
	private final BufferedImage texture;
	private final int avgColor;
	private final BufferedImage topTexture;
	private final int avgTopColor;

	Block(int id, String name, String textureName, float refractionIndex) {
		this(id, name, textureName, textureName, false, refractionIndex);
	}
	
	Block(int id, String name, String textureName, boolean solid) {
		this(id, name, textureName, textureName, solid);
	}

	Block(int id, String name, String textureName, String topTextureName, boolean solid) {
		this(id, name, textureName, topTextureName, solid, 1.333f);
	}
	
	Block(int id, String name, String textureName, String topTextureName, boolean solid, float refractionIndex) {
		this.id = id;
		this.name = name;
		this.solid = solid;
		this.refractionIndex = refractionIndex;
		
		this.texture = ImageLoader.get("tiles/"+textureName+".png");
		this.topTexture = ImageLoader.get("tiles/"+topTextureName+".png");
		
        this.avgColor = ImageTools.calcAvgColor(texture);

        this.avgTopColor = ImageTools.calcAvgColor(texture);
	}

	/**
	 * A block is solid if it occupies an entire one meter cube of space
	 * and no light can pass through it.
	 * 
	 * @return {@code true} if the block is solid
	 */
	public boolean isSolid() {
		return solid;
	}
	
	/**
	 * The name of the block.
	 * 
	 * @return
	 */
	public String getBlockName() {
		return name;
	}
	
	/**
	 * Get the texture for the block.
	 * 
	 * @return block texture
	 */
	public BufferedImage getTexture() {
		return texture;
	}
	
	/**
	 * Get the average color for the block texture as an
	 * RGB value.
	 * 
	 * @return average color (RGB)
	 */
	public int getAvgRGB() {
		return avgColor;
	}
	
	/**
	 * Get the top texture for the block.
	 * 
	 * @return block texture
	 */
	public BufferedImage getTopTexture() {
		return topTexture;
	}
	
	/**
	 * Get the average color for the block texture as an
	 * RGB value.
	 * 
	 * @return average color (RGB)
	 */
	public int getAvgTopRGB() {
		return avgTopColor;
	}
	
	@Override
	public String toString() {
		return getBlockName();
	}

    public int getColor(float u, float v) {
        return texture.getRGB(
                Math.min(texture.getWidth()-1, (int) (u * texture.getWidth())),
                Math.min(texture.getHeight()-1, (int) (v * texture.getHeight())));
    }
    
    public static final Block[] values = values();
}
