/* Copyright (c) 2010-2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.world;

import java.util.HashSet;
import java.util.Set;

/**
 * Block types. Would be nice to have block classes too.
 * 
 * @author Jesper Öqvist (jesper@llbit.se)
 */
public abstract class BlockType {

	public abstract boolean contains(int id);
	
	/**
	 * Any block a mob can spawn on. 
	 */
	public static final BlockType unoccupiedBlock = new BlockType() {
		Set<Integer> set = new HashSet<Integer>();
		{
			set.add(Block.AIR.id);
			set.add(Block.SAPLING.id);
			set.add(Block.YELLOWFLOWER.id);
			set.add(Block.REDROSE.id);
			set.add(Block.BROWNMUSHROOM.id);
			set.add(Block.REDMUSHROOM.id);
			set.add(Block.TORCH.id);
			set.add(Block.FIRE.id);
			set.add(Block.REDSTONEWIRE.id);
			set.add(Block.CROPS.id);
			set.add(Block.SIGNPOST.id);
			set.add(Block.WOODENDOOR.id);
			set.add(Block.LADDER.id);
			set.add(Block.MINECARTTRACKS.id);
			set.add(Block.WALLSIGN.id);
			set.add(Block.STONEBUTTON.id);
			set.add(Block.LEVER.id);
			set.add(Block.STONEPRESSUREPLATE.id);
			set.add(Block.IRONDOOR.id);
			set.add(Block.WOODENPRESSUREPLATE.id);
			set.add(Block.WOODENDOOR.id);
			set.add(Block.REDSTONETORCHOFF.id);
			set.add(Block.REDSTONETORCHON.id);
			set.add(Block.SNOW.id);
			set.add(Block.SUGARCANE.id);
		}
		@Override
		public boolean contains(int id) {
			return set.contains(id);
		}
	};
	
	public static final BlockType surfaceBlock = new BlockType() {
		@Override
		public boolean contains(int id) {
			return id != Block.AIR.id;
		}
	};
	
	/**
	 * Blocks which are considered part of the terrain (for heightmap
	 * purposes). This is most of the solid blocks except tree, cacti,
	 * pumpkins among other decorational blocks.
	 */
	public static final BlockType terrainBlock = new BlockType() {
		Set<Integer> set = new HashSet<Integer>();
		{
			set.add(Block.STONE.id);
			set.add(Block.GRASS.id);
			set.add(Block.DIRT.id);
			set.add(Block.COBBLESTONE.id);
			set.add(Block.WOODENPLANKS.id);
			set.add(Block.BEDROCK.id);
			set.add(Block.SAND.id);
			set.add(Block.GRAVEL.id);
			set.add(Block.GOLDORE.id);
			set.add(Block.GOLDBLOCK.id);
			set.add(Block.IRONORE.id);
			set.add(Block.IRONBLOCK.id);
			set.add(Block.COALORE.id);
			set.add(Block.SANDSTONE.id);
			set.add(Block.WOOL.id);
			set.add(Block.DOUBLESTONESLAB.id);
			set.add(Block.STONESLAB.id);
			set.add(Block.BRICKS.id);
			set.add(Block.BOOKSHELF.id);
			set.add(Block.GRASS.id);
			set.add(Block.MOSSSTONE.id);
			set.add(Block.OBSIDIAN.id);
			set.add(Block.WOODENSTAIRS.id);
			set.add(Block.CHEST.id);
			set.add(Block.DIAMONDORE.id);
			set.add(Block.DIAMONDBLOCK.id);
			set.add(Block.WORKBENCH.id);
			set.add(Block.SOIL.id);
			set.add(Block.FURNACE.id);
			set.add(Block.BURNINGFURNACE.id);
			set.add(Block.STONESTAIRS.id);
			set.add(Block.REDSTONEORE.id);
			set.add(Block.GLOWINGREDSTONEORE.id);
			set.add(Block.CLAY.id);
			set.add(Block.JUKEBOX.id);
			set.add(Block.NETHERRACK.id);
			set.add(Block.SOULSAND.id);
			set.add(Block.GLOWSTONE.id);
		}
		@Override
		public boolean contains(int id) {
			return set.contains(id);
		}
	};
	
	/**
 	 * Solid blocks are blocks which do not let any light pass through
 	 * (top-to-bottom).
 	 */
	public static final BlockType solidBlock = new BlockType() {
		Set<Integer> set = new HashSet<Integer>();
		{
			set.add(Block.STONE.id);
			set.add(Block.GRASS.id);
			set.add(Block.DIRT.id);
			set.add(Block.COBBLESTONE.id);
			set.add(Block.WOODENPLANKS.id);
			set.add(Block.WOOD.id);
			set.add(Block.BEDROCK.id);
			set.add(Block.SAND.id);
			set.add(Block.SANDSTONE.id);
			set.add(Block.GRAVEL.id);
			set.add(Block.GOLDORE.id);
			set.add(Block.GOLDBLOCK.id);
			set.add(Block.IRONORE.id);
			set.add(Block.IRONBLOCK.id);
			set.add(Block.COALORE.id);
			set.add(Block.WOOL.id);
			set.add(Block.DOUBLESTONESLAB.id);
			set.add(Block.STONESLAB.id);
			set.add(Block.BRICKS.id);
			set.add(Block.TNT.id);
			set.add(Block.BOOKSHELF.id);
			set.add(Block.MOSSSTONE.id);
			set.add(Block.OBSIDIAN.id);
			set.add(Block.WOODENSTAIRS.id);
			set.add(Block.CHEST.id);
			set.add(Block.DIAMONDORE.id);
			set.add(Block.DIAMONDBLOCK.id);
			set.add(Block.WORKBENCH.id);
			set.add(Block.SOIL.id);
			set.add(Block.FURNACE.id);
			set.add(Block.BURNINGFURNACE.id);
			set.add(Block.STONESTAIRS.id);
			set.add(Block.REDSTONEORE.id);
			set.add(Block.GLOWINGREDSTONEORE.id);
			set.add(Block.SNOW.id);
			set.add(Block.SNOWBLOCK.id);
			set.add(Block.CACTUS.id);
			set.add(Block.CLAY.id);
			set.add(Block.JUKEBOX.id);
			set.add(Block.PUMPKIN.id);
			set.add(Block.NETHERRACK.id);
			set.add(Block.SOULSAND.id);
			set.add(Block.GLOWSTONE.id);
			set.add(Block.JACKOLANTERN.id);
		}
		@Override
		public boolean contains(int id) {
			return set.contains(id);
		}
	};
	
	public static final BlockType water = new BlockType() {
		@Override
		public boolean contains(int id) {
			return id == Block.WATER.id
				|| id == Block.STATIONARYWATER.id;
		}
	};
}
