/* Copyright (c) 2010-2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.world;

import java.util.ArrayList;
import java.util.List;

public class ChunkView {

	public final double x0;
	public final double z0;
	public final double x1;
	public final double z1;
	public final int ix0;
	public final int iz0;
	public final int ix1;
	public final int iz1;
	public final int width;
	public final int height;
	public final int scale;
	private final List<List<ChunkPosition>> visible;

	
	public ChunkView() {
		this(0, 0, 0, 0, 0, 0);
	}
	
	public ChunkView(double x0, double x1, double z0, double z1, int width, int height) {
		this(x0, x1, z0, z1, width, height, 16);
	}
	
	public ChunkView(double x0, double x1, double z0, double z1, int width, int height, int scale) {
		this.x0 = x0;
		this.z0 = z0;
		this.x1 = x1;
		this.z1 = z1;
		this.width = width;
		this.height = height;
		ix0 = (int) Math.floor(x0-2);
		ix1 = (int) Math.ceil(x1+1);
		iz0 = (int) Math.floor(z0-1);
		iz1 = (int) Math.ceil(z1+2);
		this.scale = scale;
		
		visible = new ArrayList<List<ChunkPosition>>(ix1 - ix0 + 1);
		for (int x = ix0; x <= ix1; ++x) {
			List<ChunkPosition> sublist = new ArrayList<ChunkPosition>(iz1 - iz0 + 1);
			for (int z = iz0; z <= iz1; ++z) {
				sublist.add(ChunkPosition.get(x, z));
			}
			visible.add(sublist);
		}
	}

	public boolean isVisible(ChunkPosition pos) {
		return ix0 <= pos.x && ix1 >= pos.x &&
			iz0 <= pos.z && iz1 >= pos.z;
	}

	/**
	 * Get the chunk positions for chunks that are (fully and partially) visible in this view.
	 * @return a collection of the visible chunk positions
	 */
	public List<List<ChunkPosition>> visibleChunks() {
		return visible;
	}

	@Override
	public boolean equals(Object obj) {
		if (obj == this)
			return true;
		if (obj instanceof ChunkView) {
			ChunkView other = (ChunkView) obj;
			if (scale == other.scale
					&& ix0 == other.ix0
					&& ix1 == other.ix1
					&& iz0 == other.iz0
					&& iz1 == other.iz1)
				return true;
		}
		
		return false;
	}
}
