/* Copyright (c) 2010-2012 Jesper Öqvist <jesper@llbit.se>
 *
 * This file is part of Chunky.
 *
 * Chunky is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Chunky is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with Chunky.  If not, see <http://www.gnu.org/licenses/>.
 */
package se.llbit.chunky.world;

import java.io.File;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

public class WorldScanner extends Thread {

	private World world;
	
	public interface Operator {
	    void foundRegion(File worldDirectory, int x, int z);
	}
	
	private static Pattern anvilPattern = Pattern.compile("r\\.([^\\.]+)\\.([^\\.]+)\\.mca");

	public WorldScanner(World world) {
	    super("World Scanner");
		this.world = world;
	}
	
	public void run() {
		findExistingChunks(world.getWorldDirectory(), new Operator() {
            @Override
            public void foundRegion(File worldDirectory, int x, int z) {
                world.addRegion(ChunkPosition.get(x, z));
            }
        });
		world.finalizeChunkDiscovery();
	}
	
	public static void findExistingChunks(File worldDirectory, Operator operator) {
	    if (!worldDirectory.exists())
	        return;
	    
		File regionDirectory = new File(worldDirectory, "region");
		if (!regionDirectory.isDirectory()) {
			System.err.println(String.format("Failed to read region directory for world %s!",
					worldDirectory.getPath()));
			return;
		}
		
		for (File anvilFile : regionDirectory.listFiles()) {
			
			Matcher matcher = anvilPattern.matcher(anvilFile.getName());
			
			if (!anvilFile.isDirectory() && matcher.matches()) {
				
				String x = matcher.group(1);
				String z = matcher.group(2);
				if (x != null && z != null) {
				    operator.foundRegion(worldDirectory, Integer.parseInt(x), Integer.parseInt(z));
				}
			}
		}
	}
}
